Package: Statial
Type: Package
Title: A package to identify changes in cell state relative to spatial
        associations
Version: 1.15.0
Date: 2025-10-14
Authors@R: c(person("Farhan", "Ameen", email = "fame2827@uni.sydney.edu.au",
    role = c("aut", "cre")),
    person("Sourish", "Iyengar", email = "sourish.iyengar@sydney.edu.au",
    role = c("aut")),
    person("Alex", "Qin", email = "alex.qin@sydney.edu.au",
    role = c("aut")),
    person("Shila", "Ghazanfar", email = "shila.ghazanfar@sydney.edu.au",
    role = c("aut")),
    person("Ellis", "Patrick", email = "ellis.patrick@sydney.edu.au",
    role = c("aut")))
VignetteBuilder: knitr
Encoding: UTF-8
biocViews: SingleCell, Spatial, Classification
Depends: R (>= 4.1.0)
Imports: BiocParallel, spatstat.geom, concaveman, data.table,
        spatstat.explore, dplyr, tidyr, SingleCellExperiment, tibble,
        stringr, tidyselect, ggplot2, methods, stats,
        SummarizedExperiment, S4Vectors, plotly, purrr, ranger,
        magrittr, limma, SpatialExperiment, cluster, treekoR, edgeR
Suggests: BiocStyle, knitr, testthat (>= 3.0.0), ClassifyR, spicyR,
        ggsurvfit, lisaClust, survival
Description: Statial is a suite of functions for identifying changes in cell 
    state. The functionality provided by Statial provides robust quantification 
    of cell type localisation which are invariant to changes in tissue 
    structure. In addition to this Statial uncovers changes in marker expression
    associated with varying levels of localisation. These features can be used 
    to explore how the structure and function of different cell types may be 
    altered by the agents they are surrounded with.
License: GPL-3
RoxygenNote: 7.3.3
Config/testthat/edition: 3
URL: https://sydneybiox.github.io/Statial
        https://github.com/SydneyBioX/Statial/issues
BugReports: https://github.com/SydneyBioX/Statial/issues
git_url: https://git.bioconductor.org/packages/Statial
git_branch: devel
git_last_commit: 97ee04a
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:34:29 UTC; biocbuild
Author: Farhan Ameen [aut, cre],
  Sourish Iyengar [aut],
  Alex Qin [aut],
  Shila Ghazanfar [aut],
  Ellis Patrick [aut]
Maintainer: Farhan Ameen <fame2827@uni.sydney.edu.au>
